@@ -1,4 +1,4 @@
-//@line 4 "/home/daniel/trees/bg-ng/toolkit/components/url-classifier/nsUrlClassifierLib.js"
+//@line 4 "/home/glazou/trees/trunk/toolkit/components/url-classifier/nsUrlClassifierLib.js"
 
 // We wastefully reload the same JS files across components.  This puts all
 // the common JS files used by safebrowsing and url-classifier into a
@@ -10,7 +10,7 @@ const G_GDEBUG = false;
 
 Components.utils.import("resource://gre/modules/XPCOMUtils.jsm");
 
-//@line 4 "/home/daniel/trees/bg-ng/toolkit/components/url-classifier/content/moz/lang.js"
+//@line 4 "/home/glazou/trees/trunk/toolkit/components/url-classifier/content/moz/lang.js"
 
 
 /**
@@ -90,7 +90,7 @@ Function.prototype.inherits = function(parentCtor) {
   this.superClass_ = parentCtor.prototype;
   this.prototype = new tempCtor();
 }
-//@line 4 "/home/daniel/trees/bg-ng/toolkit/components/url-classifier/content/moz/preferences.js"
+//@line 4 "/home/glazou/trees/trunk/toolkit/components/url-classifier/content/moz/preferences.js"
 
 
 // Class for manipulating preferences. Aside from wrapping the pref
@@ -250,7 +250,7 @@ G_Preferences.prototype.addObserver = function(which, callback) {
     var observer = new G_PreferenceObserver(callback);
     this.observers_[which].callbacks.push(callback);
     this.observers_[which].observers.push(observer);
-    this.prefs_.addObserver(which, observer, false /* strong reference */);
+    this.prefs_.addObserver(which, observer /* strong reference */);
   }
 }
 
@@ -273,7 +273,7 @@ G_Preferences.prototype.removeObserver = function(which, callback) {
  */
 G_Preferences.prototype.removeAllObservers = function() {
   for (var which in this.observers_) {
-    for each (var observer in this.observers_[which].observers) {
+    for (var observer of this.observers_[which].observers) {
       this.prefs_.removeObserver(which, observer);
     }
   }
@@ -321,9 +321,9 @@ G_PreferenceObserver.prototype.QueryInterface = function(iid) {
   throw Components.results.NS_ERROR_NO_INTERFACE;
 }
 
-//@line 4 "/home/daniel/trees/bg-ng/toolkit/components/url-classifier/content/moz/debug.js"
+//@line 4 "/home/glazou/trees/trunk/toolkit/components/url-classifier/content/moz/debug.js"
 
-//@line 834 "/home/daniel/trees/bg-ng/toolkit/components/url-classifier/content/moz/debug.js"
+//@line 834 "/home/glazou/trees/trunk/toolkit/components/url-classifier/content/moz/debug.js"
 
 // Stubs for the debugging aids scattered through this component.
 // They will be expanded if you compile yourself a debug build.
@@ -357,7 +357,7 @@ this.G_debugService = {
     reportScriptError_: () => {}
 };
 
-//@line 4 "/home/daniel/trees/bg-ng/toolkit/components/url-classifier/content/moz/alarm.js"
+//@line 4 "/home/glazou/trees/trunk/toolkit/components/url-classifier/content/moz/alarm.js"
 
 
 // An Alarm fires a callback after a certain amount of time, or at
@@ -512,7 +512,7 @@ G_ConditionalAlarm.prototype.notify = function(timer) {
     this.cancel();
   }
 }
-//@line 4 "/home/daniel/trees/bg-ng/toolkit/components/url-classifier/content/moz/cryptohasher.js"
+//@line 4 "/home/glazou/trees/trunk/toolkit/components/url-classifier/content/moz/cryptohasher.js"
 
 
 // A very thin wrapper around nsICryptoHash. It's not strictly
@@ -655,18 +655,18 @@ G_CryptoHasher.prototype.toHex_ = function(str) {
   return hexrep.join('');
 }
 
-//@line 4 "/home/daniel/trees/bg-ng/toolkit/components/url-classifier/content/moz/observer.js"
+//@line 4 "/home/glazou/trees/trunk/toolkit/components/url-classifier/content/moz/observer.js"
 
 
 // A couple of classes to simplify creating observers. 
 //
-// // Example1:
+// Example1:
 //
 // function doSomething() { ... }
 // var observer = new G_ObserverWrapper(topic, doSomething);
 // someObj.addObserver(topic, observer);
 //
-// // Example2: 
+// Example2: 
 //
 // function doSomething() { ... }
 // new G_ObserverServiceObserver("profile-after-change", 
@@ -736,7 +736,7 @@ function G_ObserverServiceObserver(topic, observeFunction, opt_onlyOnce) {
                                          BindToObject(this.observe_, this));
   this.observerService_ = Cc["@mozilla.org/observer-service;1"]
                           .getService(Ci.nsIObserverService);
-  this.observerService_.addObserver(this.observer_, this.topic_, false);
+  this.observerService_.addObserver(this.observer_, this.topic_);
 }
 
 /**
@@ -756,7 +756,7 @@ G_ObserverServiceObserver.prototype.observe_ = function(subject, topic, data) {
     this.unregister();
 }
 
-//@line 4 "/home/daniel/trees/bg-ng/toolkit/components/url-classifier/content/moz/protocol4.js"
+//@line 4 "/home/glazou/trees/trunk/toolkit/components/url-classifier/content/moz/protocol4.js"
 
 
 // A helper class that knows how to parse from and serialize to
@@ -832,8 +832,9 @@ G_Protocol4Parser.prototype.serialize = function(map) {
   return text;
 }
 
-//@line 22 "/home/daniel/trees/bg-ng/toolkit/components/url-classifier/nsUrlClassifierLib.js"
+//@line 22 "/home/glazou/trees/trunk/toolkit/components/url-classifier/nsUrlClassifierLib.js"
 
+//@line 1 "/home/glazou/trees/trunk/toolkit/components/url-classifier/content/request-backoff.js"
 /* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
@@ -950,7 +951,7 @@ RequestBackoff.prototype.isErrorStatus = function(status) {
           HTTP_TEMPORARY_REDIRECT == status);
 }
 
-//@line 4 "/home/daniel/trees/bg-ng/toolkit/components/url-classifier/content/xml-fetcher.js"
+//@line 4 "/home/glazou/trees/trunk/toolkit/components/url-classifier/content/xml-fetcher.js"
 
 // A simple class that encapsulates a request. You'll notice the
 // style here is different from the rest of the extension; that's
@@ -984,15 +985,16 @@ this.PROT_NewXMLHttpRequest = function PROT_NewXMLHttpRequest() {
  * Note, that XMLFetcher is only used for SafeBrowsing, therefore
  * we inherit from nsILoadContext, so we can use the callbacks on the
  * channel to separate the safebrowsing cookie based on a reserved
- * appId.
+ * first-party domain.
  * @constructor
  */
 this.PROT_XMLFetcher = function PROT_XMLFetcher() {
   this.debugZone = "xmlfetcher";
   this._request = PROT_NewXMLHttpRequest();
   // implements nsILoadContext
-  this.appId = Ci.nsIScriptSecurityManager.SAFEBROWSING_APP_ID;
-  this.isInBrowserElement = false;
+  this.firstPartyDomain =
+    "safebrowsing.86868755-6b82-4842-b301-72671a0db32e.mozilla";
+  this.isInIsolatedMozBrowserElement = false;
   this.usePrivateBrowsing = false;
   this.isContent = false;
 }
@@ -1002,11 +1004,11 @@ PROT_XMLFetcher.prototype = {
    * Function that will be called back upon fetch completion.
    */
   _callback: null,
-  
+
 
   /**
    * Fetches some content.
-   * 
+   *
    * @param page URL to fetch
    * @param callback Function to call back when complete.
    */
@@ -1015,6 +1017,10 @@ PROT_XMLFetcher.prototype = {
     this._request = PROT_NewXMLHttpRequest();
     this._callback = callback;
     var asynchronous = true;
+    this._request.loadInfo.originAttributes = {
+      firstPartyDomain: this.firstPartyDomain,
+      inIsolatedMozBrowser: this.isInIsolatedMozBrowserElement
+    };
     this._request.open("GET", page, asynchronous);
     this._request.channel.notificationCallbacks = this;
 
@@ -1022,7 +1028,7 @@ PROT_XMLFetcher.prototype = {
     var self = this;
     this._request.addEventListener("readystatechange", function() {
       self.readyStateChange(self);
-    }, false);
+    });
 
     this._request.send(null);
   },
@@ -1048,7 +1054,7 @@ PROT_XMLFetcher.prototype = {
     var responseText = null;
     var status = Components.results.NS_ERROR_NOT_AVAILABLE;
     try {
-      G_Debug(this, "xml fetch status code: \"" + 
+      G_Debug(this, "xml fetch status code: \"" +
               fetcher._request.status + "\"");
       status = fetcher._request.status;
       responseText = fetcher._request.responseText;
@@ -1070,7 +1076,24 @@ PROT_XMLFetcher.prototype = {
                                          Ci.nsISupports,
                                          Ci.nsILoadContext])
 };
-//@line 25 "/home/daniel/trees/bg-ng/toolkit/components/url-classifier/nsUrlClassifierLib.js"
+//@line 25 "/home/glazou/trees/trunk/toolkit/components/url-classifier/nsUrlClassifierLib.js"
+
+// Wrap a general-purpose |RequestBackoff| to a v4-specific one
+// since both listmanager and hashcompleter would use it.
+// Note that |maxRequests| and |requestPeriod| is still configurable
+// to throttle pending requests.
+function RequestBackoffV4(maxRequests, requestPeriod) {
+  let rand = Math.random();
+  let retryInterval = Math.floor(15 * 60 * 1000 * (rand + 1));   // 15 ~ 30 min.
+  let backoffInterval = Math.floor(30 * 60 * 1000 * (rand + 1)); // 30 ~ 60 min.
+
+  return new RequestBackoff(2 /* max errors */,
+                retryInterval /* retry interval, 15~30 min */,
+                  maxRequests /* num requests */,
+                requestPeriod /* request time, 60 min */,
+              backoffInterval /* backoff interval, 60 min */,
+          24 * 60 * 60 * 1000 /* max backoff, 24hr */);
+}
 
 // Expose this whole component.
 var lib = this;
